'' =================================================================================================
''
''   File....... jm_mma7660fc_demo.spin 
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 17 AUG 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' i2c / eeprom / MMA7660
  SCL = 28


obj

' main                                                           ' * master Spin cog
  time : "jm_time"                                               '   timing and delays
  io   : "jm_io_basic"                                           '   essential io control
  acc  : "jm_mma7660fc"                                          '   MMA7660 accelerometer
  term : "jm_fullduplexserial"                                   ' * serial io for terminal

' * Uses cog when loaded


var

  long  xg
  long  yg
  long  zg
  long  tilt


dat

  Display       byte    "====================", 13
                byte    "   MMA7660FC Demo   ", 13
                byte    "====================", 13
                byte    13
                byte    "X Axis........      ", 13 
                byte    "Y Axis........      ", 13 
                byte    "Z Axis........      ", 13 
                byte    13
                byte    "Side..........      ", 13  
                byte    "Orientation...      ", 13
                byte    13
                byte    "Tap...........      ", 13
                byte    "Shake.........      ", 13
                byte    0                    

  Yes           byte    "Yes", 0
  No            byte    "No ", 0  
  

pub main | t

  setup
  
  term.rxflush                                                   ' wait open terminal & keypress
  term.rx
  term.tx(term#CLS)
  term.str(@Display)

  t := cnt
  repeat
    update_display                                               ' update display
    waitcnt(t += (MS_001 * 100))                                 '  every 100ms


pub update_display 
  
  acc.read_all(@xg)                                              '  read all axes + tilt                
  show_gforce(15, 4, xg)                                                                               
  show_gforce(15, 5, yg)                                                                               
  show_gforce(15, 6, zg)   

  show_side(15, 8, acc.side(tilt))                                                                     
  show_orientation(15, 9, acc.orientation(tilt))                                                       
                                                                                                       
  goto_xy(15, 11)                                                                                      
  tf_str(acc.tap(tilt), @Yes, @No)                               ' display tap status                  
                                                                                                       
  goto_xy(15, 12)                                                                                      
  tf_str(acc.shake(tilt), @Yes, @No)                             ' display shake status                


pub show_gforce(x, y, g)

'' Displays g-force at x,y as n.nn
'' -- g-force value expressed in 1/100g

  goto_xy(x, y)

  if (g < 0)
    term.tx("-")                                                 ' display sign
    -g                                                           ' remove sign from g-force  
  else                                                           
    term.tx(" ")

  term.tx("0" + (g / 100))                                       ' display whole g's 
  term.tx(".")
  term.tx("0" + (g / 10 // 10))                                  ' 10ths
  term.tx("0" + (g // 10))                                       ' 100ths
         

pub show_side(x, y, s)

  goto_xy(x, y)

  case s
    acc#FRONT : term.str(string("Front")) 
    acc#BACK  : term.str(string("Back "))
    other     : term.str(string("?    "))


pub show_orientation(x, y, o)

  goto_xy(x, y)

  case o
    acc#LEFT  : term.str(string("Left ")) 
    acc#RIGHT : term.str(string("Right"))
    acc#DOWN  : term.str(string("Down ")) 
    acc#UP    : term.str(string("Up   "))
    other     : term.str(string("?    "))    
 
  
pub tf_str(value, p_str1, p_str0)

'' Print true/false strings
'' -- print string at p_str1 if value <> 0
'' -- print string at p_str1 if value == 0

  if (value)
    term.str(p_str1)
  else
    term.str(p_str0)                                            
                                           

pub goto_xy(x, y)

'' Move cursor to position x,y in PST 

  term.tx(term#GOTOXY)
  term.tx(x)
  term.tx(y)
  
    
pub setup

'' Setup IO and objects for application

  time.start                                                     ' setup timing/delays for clock speed

  io.start(0, 0)                                                 ' clear all pins

  acc.start(SCL, SDA)                                            ' connect to accelerometer  

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}